import RefRegistry from './ref-registry';
export interface RefId<S> {
    __do_not_use_this_refid: S;
}
export interface RefType<T> {
    __do_not_use_this_reftype: T;
}
/**
 * A reference to an entity in a registry
 */
export declare type Ref<K extends string, S extends {} = string> = [RefType<K>, RefId<S>];
/**
 * Get a reference type for a registry
 */
export declare type RefFor<K extends keyof RefRegistry> = RefRegistry[K];
export declare type RefTypes = keyof RefRegistry;
export declare type AnyRef = RefRegistry[RefTypes];
/**
 * Check to see whether a value is a reference
 * @param thing value to check
 */
export declare function isRef<R extends Ref<any>>(thing?: R): thing is R;
/**
 * Create a new reference
 * @param type name of the reference type
 * @param id registry-unique of the reference
 * @returns the new reference
 */
export declare function createRef<K extends RefTypes>(type: K, id: string): Ref<K>;
/**
 * Get a reference's type name
 * @param ref the reference
 */
export declare function refType<K extends string>(ref: Ref<K, any>): K;
/**
 * Get a reference's ID
 * @param ref the reference
 */
export declare function refId<S extends {}>(ref: Ref<any, S>): S;
//# sourceMappingURL=ref.d.ts.map